"""GDI formulas

This script groups all formulas for GDI calculation.

This script requires that `numpy` be installed within the Python
environment you are running this script in.

This file can also be imported as a module and contains the following
functions:

    * get_control_vector - returns the control vector
    * get_delta_ave - returns the delta ave
    * get_control - returns the control
    * get_d - returns the d
    * get_ln_d - returns the ln d
    * get_standard_deviation - returns the standard deviation
    * get_mean_ln_d - returns the mean ln d
    * get_diff - returns the diff
    * get_ln_2_diff - return the ln 2 diff
    * get_z - returns the z
    * get_gdi - returns the gdi
"""

import numpy as np
import math


def get_transposed_control_feature_data(transposed_features, control_kinematics):
    return np.dot(transposed_features, control_kinematics).transpose()


def get_control_vector(control_feature_data):
    control = []
    for column in control_feature_data.T:
        control.append(get_control(column))
    return control


def get_delta_ave(control_feature_data_matrix: np.ndarray, control: list):
    delta_ave = []
    for control_feature_row in control_feature_data_matrix:
        delta_ave_row = []
        for idx, control_feature in enumerate(control_feature_row):
            if abs(control_feature) > 0:
                delta_ave_row.append(control_feature - control[idx])
            else:
                break
        if len(delta_ave_row) != 0:
            delta_ave.append(delta_ave_row)

    return delta_ave


def get_control(columns: np.ndarray):
    total = 0
    count = 0
    for row in columns:
        total += row
        if row != 0:
            count = count + 1
    return total / count


def get_d(delta_ave_matrix: list):
    d = []
    for delta_ave_row in delta_ave_matrix:
        d.append(math.sqrt(np.dot(delta_ave_row, delta_ave_row)))
    return d


def get_ln_d(d: list):
    ln_d = []
    for item in d:
        ln_d.append(np.log(item))
    return ln_d


def get_standard_deviation(ln_d: list):
    return np.std(ln_d)


def get_mean_ln_d(ln_d: list):
    return np.average(ln_d)


def get_diff(subject, controls):
    return np.array(subject) - np.array(controls)


def get_ln_2_diff(diff):
    return np.log(math.sqrt(np.dot(diff, diff)))


def get_subject(transposed_features, patient_data):
    return np.dot(transposed_features, patient_data)


def get_z(ln_2_d, mean_ln_d, std_ln_d):
    return (ln_2_d - mean_ln_d) / std_ln_d


def get_gdi(z):
    return 100 - 10 * z
