"""Input File Parser

This script parse the input file (.csv) and loads specified columns.

This tool accepts (.csv) files.

This script requires that `numpy` be installed within the Python
environment you are running this script in.

This file can also be imported as a module and contains the following
functions:

    * load - returns the list of InputData objects (represents required g values and side)
"""

import numpy as np

from .g_section import GSection
from .input_data import InputData


def _resolve_side(filename: str, csv_headers: list):
    """Resolves the side based on the input file suffix and check if the headers contain required columns for GDI

    Parameters
    ----------
    filename : str
        The input filename
    csv_headers : list
        The all CSV header in the file

    Raises
    ------
    Exception
        If the file does not contain column with side prefix.

    Returns
    -------
    out : str
        The 'R' or 'L' side
    """
    if filename.endswith('-R.csv'):
        side = 'R'
    elif filename.endswith('-L.csv'):
        side = 'L'
    else:
        raise Exception('The input filename does not contain L or R side at the end. Filename: "' + filename + '"')

    for item in GSection:
        # The column name must contain 'R' or 'L' prefix based on the filename
        required_column = side + item.input_column
        if required_column not in csv_headers:
            raise Exception('The input filename does not contain column: "' + required_column + '"')

    return side


def load(filename: str = ''):
    """Loads the input file, recognise the side and fill required columns for g list.

    Parameters
    ----------
    filename : str
        The filename

    Returns
    -------
    out: InputData
        The object that holds required values for g file
    """
    # Skip first header as it is not used
    csv_content = np.genfromtxt(filename, delimiter=',', dtype=float, encoding='UTF-8', names=True, skip_header=1)

    side = _resolve_side(filename, csv_content.dtype.names)

    print("Side is " + side)
    def resolve_name(gdi: GSection): return csv_content[gdi.get_full_column_name(side)]

    return InputData(
        side,
        pelvisAnglesX=resolve_name(GSection.PELVIS_ANGLES_X),
        pelvisAnglesY=resolve_name(GSection.PELVIS_ANGLES_Y),
        pelvisAnglesZ=resolve_name(GSection.PELVIS_ANGLES_Z),
        hipAnglesX=resolve_name(GSection.HIP_ANGLES_X),
        hipAnglesY=resolve_name(GSection.HIP_ANGLES_Y),
        hipAnglesZ=resolve_name(GSection.HIP_ANGLES_Z),
        kneeAnglesX=resolve_name(GSection.KNEE_ANGLES_X),
        ankleAnglesX=resolve_name(GSection.ANKLE_ANGLES_X),
        footProgressAnglesZ=resolve_name(GSection.FOOT_PROGRESS_ANGLES_Z)
    )
