"""Gait Deviation Index calculator

This script allows the user to calculate GDI for specified patient's CSV
file. It is assumed that the first row of the CSV file is a header
(The script ignores it), the second row must contain the column names.

This tool accepts (.csv) files.

This script requires that `numpy` and `argparse` be installed within the Python
environment you are running this script in.

This file can also be imported as a module and contains the following
functions:

    * parse_input - returns the arguments passed to the script
    * load_control_kinematics - returns the control kinematics from the file
    * load_features - returns the features from the file
    * load_g - returns the g from the file
    * calculate_gdi - returns the GDI
    * main - the main function of the script
"""

import argparse
import numpy as np
import os.path

from . import csv_util
from . import formula
from . import g_input_parser


def parse_input():
    parser = argparse.ArgumentParser(description='Gait Deviation Index')
    parser.add_argument('--subject-csv', required=True, type=str, help='A path to the CSV file with subject data')
    args = parser.parse_args()

    csv_file = args.subject_csv

    if not os.path.isfile(csv_file):
        parser.error("The specified csv file does not exists: '" + csv_file + "'")

    return args


def load_control_kinematics():
    """Loads control kinematics data from the input CSV file

    Raises
    ------
    Exception
        If the file does not exists.

    Returns
    -------
    out : ndarray
        Matrix of control kinematics
    """

    filepath = os.path.join(os.path.dirname(__file__), r'./control_kinematics.csv')
    if not os.path.isfile(filepath):
        raise Exception("The expected file 'control_kinematics.csv' not found")
    control_kinematics_csv = csv_util.load(filepath)
    # Ignore the first column
    return control_kinematics_csv[:, 1:]


def load_features():
    """Loads features data from the input CSV file

    Raises
    ------
    Exception
        If the file does not exists.

    Returns
    -------
    out : ndarray
        Matrix of features
    """

    filepath = os.path.join(os.path.dirname(__file__), r'./features.csv')
    if not os.path.isfile(filepath):
        raise Exception("The expected file 'features.csv' not found")
    features_csv = csv_util.load(filepath)
    return features_csv


def load_g(input_csv_file: str):
    """Loads g data from the input CSV file

    Parameters
    ----------
    input_csv_file : str
        A filename to load

    Returns
    -------
    out: list
        List of g values
    """

    print(f'Load data from the file: "{input_csv_file}"')
    input_data = g_input_parser.load(input_csv_file)

    reduced_g = input_data.get_g()
    print(f'The g list has "{len(reduced_g)}" after the reduction')

    return reduced_g


def calculate_gdi(control_kinematics: np.ndarray, features: np.ndarray, g: list):
    """Calculates the GDI

    Parameters
    ----------
    control_kinematics : np.ndarray
        Control kinematics
    features: np.ndarray
        Transposed features
    g : list
        g data

    Returns
    -------
    out: float
        GDI value
    """
    transposed_features = features.transpose()

    control_feature_data = formula.get_transposed_control_feature_data(transposed_features, control_kinematics)

    control = formula.get_control_vector(control_feature_data)

    delta_ave = formula.get_delta_ave(control_feature_data, control)

    d = formula.get_d(delta_ave)

    ln_d = formula.get_ln_d(d)

    std_ln_d = formula.get_standard_deviation(ln_d)

    mean_ln_d = formula.get_mean_ln_d(ln_d)

    subject = formula.get_subject(transposed_features, g)

    diff = formula.get_diff(subject, control)

    ln_2_d = formula.get_ln_2_diff(diff)

    z = formula.get_z(ln_2_d, mean_ln_d, std_ln_d)

    return formula.get_gdi(z)


def main():
    args = parse_input()

    # Load constant files from the current path
    control_kinematics = load_control_kinematics()
    features = load_features()

    # Load g data from the specified CSV file
    g = load_g(args.subject_csv)

    # Get GDI result
    gdi_result = calculate_gdi(control_kinematics, features, g)
    print("GDI:", gdi_result)


if __name__ == "__main__":
    main()
